#include <Wire.h>
#include <LSM303.h>

LSM303 compass;

void setup() {
  Serial.begin(9600);
  Wire.begin();
  compass.init();
  compass.enableDefault();
  
  /*
  Calibration values; the default values of +/-32767 for each axis
  lead to an assumed magnetometer bias of 0. Use the Calibrate example
  program to determine appropriate values for your particular unit.
  */
  compass.m_min = (LSM303::vector<int16_t>) { -1980,  -2435,  -2490};

  compass.m_max = (LSM303::vector<int16_t>){ +2909,  +2291,  +2046};
}

void loop() {
  compass.read();
  
  /*
  When given no arguments, the heading() function returns the angular
  difference in the horizontal plane between a default vector and
  north, in degrees.
  
  The default vector is chosen by the library to point along the
  surface of the PCB, in the direction of the top of the text on the
  silkscreen. This is the +X axis on the Pololu LSM303D carrier and
  the -Y axis on the Pololu LSM303DLHC, LSM303DLM, and LSM303DLH
  carriers.
  
  To use a different vector as a reference, use the version of heading()
  that takes a vector argument; for example, use
  
    compass.heading((LSM303::vector<int>){0, 0, 1});
  
  to use the +Z axis as a reference.
  */
  float heading = compass.heading((LSM303::vector<int>){1, 0, 0});

  LSM303::vector<int32_t> temp_m = {compass.m.x, compass.m.y, compass.m.z};

  // subtract offset (average of min and max) from magnetometer readings
  temp_m.x -= ((int32_t)compass.m_min.x + compass.m_max.x) / 2;
  temp_m.y -= ((int32_t)compass.m_min.y + compass.m_max.y) / 2;
  temp_m.z -= ((int32_t)compass.m_min.z + compass.m_max.z) / 2;
    
  float p = pitch();
  float r = roll();

  float cmx = temp_m.x * cos(p)
            + temp_m.y * sin(r) * sin(p)
            + temp_m.z * cos(r) * sin(p);
  float cmy = temp_m.y * cos(r)
            - temp_m.z * sin(r);

  float h2 = atan2(cmy, cmx) * 180 / PI;
  if (h2 < 0) h2 += 360;
  
  Serial.print(h2);
  Serial.print(" ");
  Serial.println(heading);
  
  /*Serial.print("pitch: ");
  Serial.print(p * 180 / PI );
  Serial.print("   roll: ");
  Serial.println(r * 180 / PI);*/
  
  delay(100);
}

float pitch()
{
  return atan2(-compass.a.x, sqrt((int32_t)compass.a.y * compass.a.y + (int32_t)compass.a.z * compass.a.z));
}

float roll()
{
  return atan2(compass.a.y, compass.a.z);
}

